//
// $Id: MapView.h,v 1.6 1997/10/31 04:51:52 nygard Exp $
//

//
//  This file is a part of Empire, a game of exploration and conquest.
//  Copyright (C) 1996  Steve Nygard
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  You may contact the author by:
//     e-mail:  nygard@telusplanet.net
//

#import <AppKit/AppKit.h>
#import "Empire.h"

@class Map;

//======================================================================

@interface MapView : NSView
{
    Map *map;
    id delegate;

    // If enabled, only sends mouse down events when the location has changed.
    BOOL compressEvents;

    // Cursor handling:
    BOOL cursorEnabled;
    BOOL cursorOn;
    BOOL fastCursor;
    EMMapLocation cursorLocation;
    NSTimer *cursorTimer;
    NSImage *cursorImage;

    Icon cursorIcon;
    Player cursorPlayer;
}

+ (void) initialize;

- initWithFrame:(NSRect)frameRect;
- (void) dealloc;

- (BOOL) acceptsFirstResponder;
- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent;

- (BOOL) isOpaque;
- (void) drawRect:(NSRect)rect;

- (Map *) map;
- (void) setMap:(Map *)aMap;



- (EMMapLocation) getLocationForPoint:(NSPoint)point;
- (NSPoint) getPointForLocation:(EMMapLocation)target;
- (NSPoint) getCenterPointForLocation:(EMMapLocation)target;
- (NSRect) getRectForLocation:(EMMapLocation)target;
- (NSRect) getRectAround3x3Location:(EMMapLocation)target;

- (void) centerLocation:(EMMapLocation)target ifNotVisible:(BOOL)notVisibleFlag;
- (void) scrollLocationToVisible:(EMMapLocation)target;

- (void) displayLocation:(EMMapLocation)target;
- (void) displayAround3x3Location:(EMMapLocation)target;



- (void) mouseDown:(NSEvent *)theEvent;
- (void) mouseUp:(NSEvent *)theEvent;
- (void) rightMouseDown:(NSEvent *)theEvent;
- (void) rightMouseUp:(NSEvent *)theEvent;
- (void) keyDown:(NSEvent *)theEvent;

- (void) enableCompressedEvents:(BOOL)flag;

- (void) vendorImagesUpdated:(BOOL)player1:(BOOL)player2:(BOOL)player3:(BOOL)other;
- (void) updateVisiblePlayer:(Player)number;

//======================================================================
// Map Observer
//======================================================================

- (void) refreshMap;
- (void) refreshLocation:(EMMapLocation)target;
- (void) refresh3x3Location:(EMMapLocation)target;

//======================================================================
// Combat report...
//======================================================================

- (void) showExplosions:(int)count atLocation:(EMMapLocation)target;

//======================================================================
// Cursor Handling
//======================================================================

- (void) centerScreenAroundCursor;
- (void) centerCursor;

- (BOOL) cursorEnabled;
- (void) setCursorEnabled:(BOOL)flag;
- (void) setFastCursor:(BOOL)flag;

- (EMMapLocation) cursorLocation;
- (void) positionCursorAtLocation:(EMMapLocation)target;

- (void) setCursor:(Icon)icon player:(Player)aPlayer;
- (void) updateCursorImage;

- (void) cursorTimer:(NSTimer *)aTimer;
- (void) updateCursor:(BOOL)scroll;

- (void) useDefaultCursor;
- (void) useDirectionCursor;

@end
